@@ -127,6 +127,11 @@ class User(aegis.database.Row):
         sql = 'UPDATE '+self.table_name+' SET member_id=%s WHERE user_id=%s'
         return db().execute(sql, member_id, self['user_id'])
 
+    def set_preferences(self, preferences):
+        preferences = json.dumps(preferences, cls=aegis.stdlib.DateTimeEncoder)
+        sql = "UPDATE "+self.table_name+" SET preferences=%s WHERE user_id=%s"
+        return db().execute(sql, preferences, self['user_id'])
+
 
 class Email(aegis.database.Row):
     table_name = 'email'
@@ -165,6 +170,11 @@ class Email(aegis.database.Row):
         sql = "UPDATE email SET google_user_id=%s WHERE email_id=%s"
         return db().execute(sql, google_user_id, self['email_id'])
 
+    def deidentify(self, email_addr):
+        # overwrite email with email_addr, stop referencing google_user.google_user_id, and mark deleted
+        sql = "UPDATE email SET email=%s, google_user_id=NULL, delete_dttm=NOW() WHERE email_id=%s"
+        return db().execute(sql, email_addr, self['email_id'])
+
 
 class Member(aegis.database.Row):
     table_name = 'member'
@@ -178,10 +188,13 @@ class Member(aegis.database.Row):
         return db().execute(sql, email_id)
 
     @classmethod
-    def set_member(cls, email_id):
+    def set_member(cls, email_id, marketing_id=None):
         member = cls.get_email_id(email_id)
         if not member:
-            member_id = cls.insert(email_id)
+            columns = {'email_id': email_id}
+            if marketing_id:
+                columns['marketing_id'] = marketing_id
+            member_id = cls.insert_columns(**columns)
             member = cls.get_id(member_id)
         return member
 
@@ -210,6 +223,11 @@ class Member(aegis.database.Row):
             member['picture_url'] = member['google_user']['picture_url']
         return member
 
+    def deidentify(self):
+        # stop referencing google_user.google_user_id, and mark deleted
+        sql = "UPDATE member SET google_user_id=NULL, delete_dttm=NOW() WHERE member_id=%s"
+        return db().execute(sql, self['member_id'])
+
 
 class MemberAuth(aegis.database.Row):
     table_name = 'member_auth'
@@ -290,6 +308,15 @@ class GoogleUser(aegis.database.Row):
         sql = "SELECT * FROM google_user WHERE google_id=%s"
         return db().get(sql, google_id, cls=cls)
 
+    @classmethod
+    def get_member_id(cls, member_id):
+        sql = "SELECT * FROM google_user WHERE member_id=%s"
+        return db().get(sql, member_id, cls=cls)
+
+    def hard_delete(self):
+        sql = "DELETE FROM google_user WHERE google_user_id=%s"
+        return db().execute(sql, self['google_user_id'])
+
 
 class GoogleAccess(aegis.database.Row):
     table_name = 'google_access'
@@ -308,6 +335,10 @@ class GoogleAccess(aegis.database.Row):
         sql = "DELETE FROM google_access WHERE expire_dttm < NOW()"
         return dbconn.execute(sql)
 
+    def hard_delete(self):
+        sql = "DELETE FROM google_access WHERE google_access_id=%s"
+        return db().execute(sql, self['google_access_id'])
+
 
 class GooglePicture(aegis.database.Row):
     table_name = 'google_picture'
@@ -321,6 +352,10 @@ class GooglePicture(aegis.database.Row):
         sql = "SELECT * FROM google_picture WHERE google_user_id=%s"
         return dbconn.get(sql, google_user_id, cls=cls)
 
+    def hard_delete(self):
+        sql = "DELETE FROM google_picture WHERE google_picture_id=%s"
+        return db().execute(sql, self['google_picture_id'])
+
 
 class EmailType(aegis.database.Row):
     table_name = 'email_type'
@@ -921,3 +956,13 @@ class Monitor(aegis.database.Row):
     def get_host_cmd(cls, host, cmd):
         sql = "SELECT * FROM monitor WHERE monitor_host=%s AND monitor_cmd=%s ORDER BY monitor_id DESC LIMIT 1"
         return db().get(sql, host, cmd, cls=cls)
+
+
+class Marketing(aegis.database.Row):
+    table_name = 'marketing'
+    id_column = 'marketing_id'
+
+    @classmethod
+    def get_name(cls, name):
+        sql = 'SELECT * FROM marketing WHERE marketing_name=%s'
+        return db().get(sql, name, cls=cls)
