@@ -203,16 +203,19 @@ class AegisHandler(tornado.web.RequestHandler):
                 user_agent = self.models['UserAgent'].get_id(user_agent['user_agent_id'])
             user = self.models['User'].get_id(user_agent['robot_user_id'])
             user_ck = {}
+            self.tmpl['user_row'] = user
         else:
             # Check if the cookie exists, and if the record exists. If either doesn't exist, start a new user record.
             user_ck = self.cookie_get('user')
             if user_ck and user_ck.get('user_id'):
                 user = self.models['User'].get_id(user_ck['user_id'])
+                self.tmpl['user_row'] = user
                 if user:
                     self.cookie_set('user', user_ck)
             if not user:
                 user_id = self.models['User'].insert(user_agent['user_agent_id'])
                 user = self.models['User'].get_id(user_id)
+                self.tmpl['user_row'] = user
                 if user_ck:
                     user_ck['user_id'] = user_id
                 else:
@@ -223,6 +226,7 @@ class AegisHandler(tornado.web.RequestHandler):
             self.tmpl['user']['user_id'] = user['user_id']
         # At the end leave the row behind for later use in the request
         self.tmpl['user_agent_row'] = user_agent
+        self.tmpl['show_cookie_banner'] = not (self.tmpl.get('user_row') and self.tmpl['user_row'].get('preferences') and self.tmpl['user_row']['preferences'].get('cookie_ok_dttm'))
 
     def enforce_admin(self):
         if not self.is_super_admin():
@@ -296,12 +300,31 @@ class AegisHandler(tornado.web.RequestHandler):
 
     def get_next_url(self, default_url='/'):
         next_url = self.get_argument('next', None)
-        if next_url:
-            return tornado.escape.url_unescape(next_url)
-        if self.tmpl['referer']:
-            return tornado.escape.url_unescape(self.tmpl['referer'])
+        url_parts = aegis.stdlib.validate_url(next_url)
+        if not url_parts:
+            return default_url
+        url_str = urllib.parse.urlunparse(url_parts)
+        if url_str:
+            return tornado.escape.url_unescape(url_str)
         return default_url
 
+    def get_url_next(self, url):
+        # Validate and parse uri
+        url_parts = urllib.parse.urlparse(url)
+        url_parts = url_parts._replace(scheme='https', netloc=self.tmpl['domain'])
+        url_str = urllib.parse.urlunparse(url_parts)
+        url_parts = aegis.stdlib.validate_url(url_str)
+        if not url_parts:
+            return
+        # Remove ?next= param in query string to prevent looping
+        query_dict = urllib.parse.parse_qs(url_parts.query)
+        if 'next' in query_dict:
+            del query_dict['next']
+        query_params = urllib.parse.urlencode(query_dict, doseq=True)
+        url_parts = url_parts._replace(query=query_params)
+        next_url = urllib.parse.urlunparse(url_parts)
+        return next_url
+
 
     # Cookie Handling
     def cookie_encode(self, val):
@@ -692,9 +715,31 @@ class AegisHandler(tornado.web.RequestHandler):
         return self.audit_session.get('marketing_id')
 
     def set_marketing_id(self):
+        # This runs before self.audit_session gets initialized from database, but skip if session_ck has audit_session_id.
+        if self.tmpl.get('session_ck', {}).get('audit_session_id'):
+            return
+        # If there's an mid in request args, use if it's in the marketing table
         mid = aegis.stdlib.validate_int(self.request.args.get('mid'))
         if mid:
-            self.audit_session['marketing_id'] = mid
+            marketing = aegis.model.Marketing.get_id(mid)
+            if not marketing:
+                mid = None
+        # Without explicit mid, infer from Referer header
+        if not mid:
+            marketing_name = 'direct'
+            referer = self.request.headers.get('Referer')
+            if referer:
+                marketing_name = 'referral'
+                url_parts = urllib.parse.urlparse(referer)
+                organic_netlocs = ['google.com', 'www.google.com', 'www.aol.com', 'aol.com', 'www.ask.com', 'ask.com', 'baidu.com', 'www.bing.com', 'bing.com',
+                                   'www.daum.net', 'daum.net', 'duckduckgo.com', 'ecosia.org', 'search.brave.com', 'www.ecosia.org', 'www.lycos.com', 'www.msn.com',
+                                   'www.yahoo.com', 'm.yahoo.com', 'yahoo.com', 'yandex.ru', 'www.yandex.com']
+                if url_parts.netloc in organic_netlocs:
+                    marketing_name = 'organic'
+            marketing = aegis.model.Marketing.get_name(marketing_name)
+            if marketing:
+                mid = marketing['marketing_id']
+        self.audit_session['marketing_id'] = mid
 
 
 class JsonRestApi(AegisHandler):
@@ -1095,7 +1140,7 @@ class AegisReportForm(AegisWeb):
             sql_error = [str(arg) for arg in ex.args]
             self.tmpl['errors']['sql_error'] = ': '.join(sql_error)
             return self.screen()
-        return self.redirect('/aegis/report/%s' % report_type_id)
+        return self.redirect('/admin/report/%s' % report_type_id)
 
     def screen(self):
         self.tmpl['schemas'] = []
